@@ -18,7 +18,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 #include <dbus/dbus-glib.h>
@@ -471,7 +471,7 @@ g_proxy_list_free (DBusGProxyList *list)
 }
 
 static char*
-g_proxy_get_match_rule (DBusGProxy *proxy)
+g_proxy_get_signal_match_rule (DBusGProxy *proxy)
 {
   DBusGProxyPrivate *priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
   /* FIXME Escaping is required here */
@@ -484,6 +484,21 @@ g_proxy_get_match_rule (DBusGProxy *proxy)
                             priv->path, priv->interface);
 }
 
+static char *
+g_proxy_get_owner_match_rule (DBusGProxy *proxy)
+{
+  DBusGProxyPrivate *priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
+  if (priv->name) 
+    {
+      return g_strdup_printf ("type='signal',sender='" DBUS_SERVICE_DBUS
+        "',path='" DBUS_PATH_DBUS
+        "',interface='" DBUS_INTERFACE_DBUS
+        "',member='NameOwnerChanged'"
+        ",arg0='%s'", priv->name);
+    }
+  return NULL;
+}
+
 typedef struct
 {
   char *name;
@@ -738,9 +753,16 @@ dbus_g_proxy_manager_replace_name_owner (DBusGProxyManager  *manager,
 	  
 	      names = g_slist_delete_link (names, link);
 
-	      if (names == NULL)
-	        g_hash_table_remove (manager->owner_names, prev_owner);
-	    }
+              if (names == NULL)
+                {
+                  g_hash_table_remove (manager->owner_names, prev_owner);
+                }
+              else
+                {
+                  g_hash_table_insert (manager->owner_names,
+                                       g_strdup (prev_owner), names);
+                }
+            }
         }
 
       if (new_owner[0] == '\0')
@@ -910,18 +932,6 @@ dbus_g_proxy_manager_register (DBusGProxyManager *manager,
                                                     g_str_equal,
                                                     g_free,
                                                     NULL);
-      /* FIXME - for now we listen for all NameOwnerChanged; once
-       * Anders' detail patch lands we should add individual rules
-       * 
-       * NOTE: if you change this, be sure to change the matching
-       * call to dbus_bus_remove_match in dbus_g_proxy_manager_unregister.
-       */
-      dbus_bus_add_match (manager->connection,
-                          "type='signal',sender='" DBUS_SERVICE_DBUS
-			  "',path='" DBUS_PATH_DBUS
-			  "',interface='" DBUS_INTERFACE_DBUS
-			  "',member='NameOwnerChanged'",
-			  NULL);
     }
   else
     {
@@ -944,20 +954,24 @@ dbus_g_proxy_manager_register (DBusGProxyManager *manager,
 
   if (list->proxies == NULL && priv->name)
     {
-      /* We have to add the match rule to the server,
+      /* We have to add match rules to the server,
        * but only if the server is a message bus,
        * not if it's a peer.
        */
        char *rule;
        
-       rule = g_proxy_get_match_rule (proxy);
-       
+       rule = g_proxy_get_signal_match_rule (proxy);
        /* We don't check for errors; it's not like anyone would handle them, and
         * we don't want a round trip here.
         */
        dbus_bus_add_match (manager->connection,
 			   rule, NULL);
+       g_free (rule);
        
+       rule = g_proxy_get_owner_match_rule (proxy);
+       if (rule)
+         dbus_bus_add_match (manager->connection,
+                             rule, NULL);
        g_free (rule);
     }
 
@@ -1061,27 +1075,21 @@ dbus_g_proxy_manager_unregister (DBusGProxyManager *manager,
                            tri);
       list = NULL;
 
-      rule = g_proxy_get_match_rule (proxy);
+      rule = g_proxy_get_signal_match_rule (proxy);
       dbus_bus_remove_match (manager->connection,
                              rule, NULL);
       g_free (rule);
+      rule = g_proxy_get_owner_match_rule (proxy);
+      if (rule)
+        dbus_bus_remove_match (manager->connection,
+                               rule, NULL);
+      g_free (rule);      
     }
   
   if (g_hash_table_size (manager->proxy_lists) == 0)
     {
       g_hash_table_destroy (manager->proxy_lists);
       manager->proxy_lists = NULL;
-
-      /*
-       * NOTE: if you change this, be sure to change the matching
-       * call to dbus_bus_add_match in dbus_g_proxy_manager_register.
-       */
-      dbus_bus_remove_match (manager->connection,
-                             "type='signal',sender='" DBUS_SERVICE_DBUS
-			     "',path='" DBUS_PATH_DBUS
-			     "',interface='" DBUS_INTERFACE_DBUS
-			     "',member='NameOwnerChanged'",
-			     NULL);
     }
 
   g_free (tri);
@@ -2410,7 +2418,7 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
  *
  * Asynchronously invokes a method on a remote interface. The method
  * call will not be sent over the wire until the application returns
- * to the main loop, or blocks in dbus_connection_flush() to write out
+ * to the main loop, or blocks in dbus_g_connection_flush() to write out
  * pending data.  The call will be completed after a timeout, or when
  * a reply is received.  When the call returns, the callback specified
  * will be invoked; you can then collect the results of the call
@@ -2463,7 +2471,7 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
  *
  * Asynchronously invokes a method on a remote interface. The method
  * call will not be sent over the wire until the application returns
- * to the main loop, or blocks in dbus_connection_flush() to write out
+ * to the main loop, or blocks in dbus_g_connection_flush() to write out
  * pending data.  The call will be completed after a timeout, or when
  * a reply is received.  When the call returns, the callback specified
  * will be invoked; you can then collect the results of the call
@@ -2732,11 +2740,10 @@ dbus_g_proxy_cancel_call (DBusGProxy        *proxy,
   call_id = DBUS_G_PROXY_CALL_TO_ID (call);
 
   pending = g_hash_table_lookup (priv->pending_calls, GUINT_TO_POINTER (call_id));
+  g_hash_table_remove (priv->pending_calls, GUINT_TO_POINTER (call_id));
   g_return_if_fail (pending != NULL);
 
   dbus_pending_call_cancel (pending);
-
-  g_hash_table_remove (priv->pending_calls, GUINT_TO_POINTER (call_id));
 }
 
 /**
@@ -2748,7 +2755,7 @@ dbus_g_proxy_cancel_call (DBusGProxy        *proxy,
  * Sends a message to the interface we're proxying for.  Does not
  * block or wait for a reply. The message is only actually written out
  * when you return to the main loop or block in
- * dbus_connection_flush().
+ * dbus_g_connection_flush().
  *
  * The message is modified to be addressed to the target interface.
  * That is, a destination name field or whatever is needed will be
